Struct gpp_solver::reexported::Pin 1.33.0[−][src]
#[repr(transparent)]pub struct Pin<P> { /* fields omitted */ }Expand description
A pinned pointer.
This is a wrapper around a kind of pointer which makes that pointer “pin” its
value in place, preventing the value referenced by that pointer from being moved
unless it implements Unpin.
See the pin module documentation for an explanation of pinning.
Implementations
Unwraps this Pin<P> returning the underlying pointer.
This requires that the data inside this Pin is Unpin so that we
can ignore the pinning invariants when unwrapping it.
Construct a new Pin<P> around a reference to some data of a type that
may or may not implement Unpin.
If pointer dereferences to an Unpin type, Pin::new should be used
instead.
Safety
This constructor is unsafe because we cannot guarantee that the data
pointed to by pointer is pinned, meaning that the data will not be moved or
its storage invalidated until it gets dropped. If the constructed Pin<P> does
not guarantee that the data P points to is pinned, that is a violation of
the API contract and may lead to undefined behavior in later (safe) operations.
By using this method, you are making a promise about the P::Deref and
P::DerefMut implementations, if they exist. Most importantly, they
must not move out of their self arguments: Pin::as_mut and Pin::as_ref
will call DerefMut::deref_mut and Deref::deref on the pinned pointer
and expect these methods to uphold the pinning invariants.
Moreover, by calling this method you promise that the reference P
dereferences to will not be moved out of again; in particular, it
must not be possible to obtain a &mut P::Target and then
move out of that reference (using, for example mem::swap).
For example, calling Pin::new_unchecked on an &'a mut T is unsafe because
while you are able to pin it for the given lifetime 'a, you have no control
over whether it is kept pinned once 'a ends:
use std::mem;
use std::pin::Pin;
fn move_pinned_ref<T>(mut a: T, mut b: T) {
unsafe {
let p: Pin<&mut T> = Pin::new_unchecked(&mut a);
// This should mean the pointee `a` can never move again.
}
mem::swap(&mut a, &mut b);
// The address of `a` changed to `b`'s stack slot, so `a` got moved even
// though we have previously pinned it! We have violated the pinning API contract.
}A value, once pinned, must remain pinned forever (unless its type implements Unpin).
Similarly, calling Pin::new_unchecked on an Rc<T> is unsafe because there could be
aliases to the same data that are not subject to the pinning restrictions:
use std::rc::Rc;
use std::pin::Pin;
fn move_pinned_rc<T>(mut x: Rc<T>) {
let pinned = unsafe { Pin::new_unchecked(Rc::clone(&x)) };
{
let p: Pin<&T> = pinned.as_ref();
// This should mean the pointee can never move again.
}
drop(pinned);
let content = Rc::get_mut(&mut x).unwrap();
// Now, if `x` was the only reference, we have a mutable reference to
// data that we pinned above, which we could use to move it as we have
// seen in the previous example. We have violated the pinning API contract.
}Gets a pinned shared reference from this pinned pointer.
This is a generic method to go from &Pin<Pointer<T>> to Pin<&T>.
It is safe because, as part of the contract of Pin::new_unchecked,
the pointee cannot move after Pin<Pointer<T>> got created.
“Malicious” implementations of Pointer::Deref are likewise
ruled out by the contract of Pin::new_unchecked.
Unwraps this Pin<P> returning the underlying pointer.
Safety
This function is unsafe. You must guarantee that you will continue to
treat the pointer P as pinned after you call this function, so that
the invariants on the Pin type can be upheld. If the code using the
resulting P does not continue to maintain the pinning invariants that
is a violation of the API contract and may lead to undefined behavior in
later (safe) operations.
If the underlying data is Unpin, Pin::into_inner should be used
instead.
Gets a pinned mutable reference from this pinned pointer.
This is a generic method to go from &mut Pin<Pointer<T>> to Pin<&mut T>.
It is safe because, as part of the contract of Pin::new_unchecked,
the pointee cannot move after Pin<Pointer<T>> got created.
“Malicious” implementations of Pointer::DerefMut are likewise
ruled out by the contract of Pin::new_unchecked.
This method is useful when doing multiple calls to functions that consume the pinned type.
Example
use std::pin::Pin;
impl Type {
fn method(self: Pin<&mut Self>) {
// do something
}
fn call_method_twice(mut self: Pin<&mut Self>) {
// `method` consumes `self`, so reborrow the `Pin<&mut Self>` via `as_mut`.
self.as_mut().method();
self.as_mut().method();
}
}Constructs a new pin by mapping the interior value.
For example, if you wanted to get a Pin of a field of something,
you could use this to get access to that field in one line of code.
However, there are several gotchas with these “pinning projections”;
see the pin module documentation for further details on that topic.
Safety
This function is unsafe. You must guarantee that the data you return will not move so long as the argument value does not move (for example, because it is one of the fields of that value), and also that you do not move out of the argument you receive to the interior function.
Gets a shared reference out of a pin.
This is safe because it is not possible to move out of a shared reference.
It may seem like there is an issue here with interior mutability: in fact,
it is possible to move a T out of a &RefCell<T>. However, this is
not a problem as long as there does not also exist a Pin<&T> pointing
to the same data, and RefCell<T> does not let you create a pinned reference
to its contents. See the discussion on “pinning projections” for further
details.
Note: Pin also implements Deref to the target, which can be used
to access the inner value. However, Deref only provides a reference
that lives for as long as the borrow of the Pin, not the lifetime of
the Pin itself. This method allows turning the Pin into a reference
with the same lifetime as the original Pin.
Converts this Pin<&mut T> into a Pin<&T> with the same lifetime.
Gets a mutable reference to the data inside of this Pin.
This requires that the data inside this Pin is Unpin.
Note: Pin also implements DerefMut to the data, which can be used
to access the inner value. However, DerefMut only provides a reference
that lives for as long as the borrow of the Pin, not the lifetime of
the Pin itself. This method allows turning the Pin into a reference
with the same lifetime as the original Pin.
Gets a mutable reference to the data inside of this Pin.
Safety
This function is unsafe. You must guarantee that you will never move
the data out of the mutable reference you receive when you call this
function, so that the invariants on the Pin type can be upheld.
If the underlying data is Unpin, Pin::get_mut should be used
instead.
Construct a new pin by mapping the interior value.
For example, if you wanted to get a Pin of a field of something,
you could use this to get access to that field in one line of code.
However, there are several gotchas with these “pinning projections”;
see the pin module documentation for further details on that topic.
Safety
This function is unsafe. You must guarantee that the data you return will not move so long as the argument value does not move (for example, because it is one of the fields of that value), and also that you do not move out of the argument you receive to the interior function.
pub fn static_ref(r: &'static T) -> Pin<&'static T>ⓘ
🔬 This is a nightly-only experimental API. (pin_static_ref)
pub fn static_ref(r: &'static T) -> Pin<&'static T>ⓘ
pin_static_ref)Get a pinned reference from a static reference.
This is safe, because T is borrowed for the 'static lifetime, which
never ends.
pub fn as_deref_mut(self) -> Pin<&'a mut <P as Deref>::Target>ⓘ
🔬 This is a nightly-only experimental API. (pin_deref_mut)
pub fn as_deref_mut(self) -> Pin<&'a mut <P as Deref>::Target>ⓘ
pin_deref_mut)Gets a pinned mutable reference from this nested pinned pointer.
This is a generic method to go from Pin<&mut Pin<Pointer<T>>> to Pin<&mut T>. It is
safe because the existence of a Pin<Pointer<T>> ensures that the pointee, T, cannot
move in the future, and this method does not enable the pointee to move. “Malicious”
implementations of P::DerefMut are likewise ruled out by the contract of
Pin::new_unchecked.
pub fn static_mut(r: &'static mut T) -> Pin<&'static mut T>ⓘ
🔬 This is a nightly-only experimental API. (pin_static_ref)
pub fn static_mut(r: &'static mut T) -> Pin<&'static mut T>ⓘ
pin_static_ref)Get a pinned mutable reference from a static mutable reference.
This is safe, because T is borrowed for the 'static lifetime, which
never ends.