Struct gpp_solver::FragmentId [−][src]
pub struct FragmentId(pub usize);Expand description
ID of a fragment.
Tuple Fields
0: usizeTrait Implementations
Performs the conversion.
Performs the conversion.
This method tests for self and other values to be equal, and is used
by ==. Read more
This method tests for !=.
This method returns an ordering between self and other values if one exists. Read more
This method tests less than (for self and other) and is used by the < operator. Read more
This method tests less than or equal to (for self and other) and is used by the <=
operator. Read more
This method tests greater than (for self and other) and is used by the > operator. Read more